@@ -34,6 +34,7 @@ var msgWindowContractID      = "@mozilla.org/messenger/msgwindow;1";
 
 var messenger;
 var pref;
+var prefServices;
 var statusFeedback;
 var messagePaneController;
 var msgWindow;
@@ -77,13 +78,17 @@ function OnMailWindowUnload()
 
   var searchSession = GetSearchSession();
   if (searchSession)
+  {
     removeGlobalListeners();
-
+    if (gPreQuickSearchView)     //close the cached pre quick search view
+      gPreQuickSearchView.close();
+  }
+  
   var dbview = GetDBView();
   if (dbview) {
     dbview.close(); 
   }
-    
+
   var mailSession = Components.classes[mailSessionContractID].getService();
   if(mailSession)
   {
@@ -108,13 +113,19 @@ function OnMailWindowUnload()
 
 }
 
+function CreateMessenger()
+{
+  messenger = Components.classes[messengerContractID].createInstance();
+  messenger = messenger.QueryInterface(Components.interfaces.nsIMessenger);
+}
+
 function CreateMailWindowGlobals()
 {
   // get the messenger instance
-  messenger = Components.classes[messengerContractID].createInstance();
-  messenger = messenger.QueryInterface(Components.interfaces.nsIMessenger);
+  CreateMessenger();
 
-  pref = Components.classes[prefContractID].getService(Components.interfaces.nsIPrefBranch);
+  prefServices = Components.classes[prefContractID].getService(Components.interfaces.nsIPrefService);
+  pref = prefServices.getBranch(null);
 
   //Create windows status feedback
   // set the JS implementation of status feedback before creating the c++ one..
@@ -246,7 +257,6 @@ function nsMsgStatusFeedback()
 nsMsgStatusFeedback.prototype =
 {
   // global variables for status / feedback information....
-  startTime  : 0,
   statusTextFld : null,
   statusBar     : null,
   throbber      : null,
@@ -278,7 +288,8 @@ nsMsgStatusFeedback.prototype =
   QueryInterface : function(iid)
     {
       if (iid.equals(Components.interfaces.nsIMsgStatusFeedback) ||
-          iid.equals(Components.interfaces.nsIXULBrowserWindow))
+          iid.equals(Components.interfaces.nsIXULBrowserWindow) ||
+          iid.equals(Components.interfaces.nsISupports))
         return this;
       throw Components.results.NS_NOINTERFACE;
     },
@@ -306,9 +317,6 @@ nsMsgStatusFeedback.prototype =
 
       //turn on stop button and menu
     this.stopCmd.removeAttribute("disabled");
-
-      // Remember when loading commenced.
-      this.startTime = (new Date()).getTime();
     },
   startMeteors : function()
     {
@@ -334,10 +342,7 @@ nsMsgStatusFeedback.prototype =
         gTimelineService.resetTimer("FolderLoading");
       }
       this.ensureStatusFields();
-      // Record page loading time.
-      var elapsed = ( (new Date()).getTime() - this.startTime ) / 1000;
-      var msg = gMessengerBundle.getFormattedString("documentDoneTime",
-                                                    [ elapsed ]);
+      var msg = gMessengerBundle.getString("documentDone");
       this.showStatusString(msg);
       defaultStatus = msg;
 
@@ -396,10 +401,10 @@ nsMsgWindowCommands.prototype =
 {
   QueryInterface : function(iid)
   {
-    if(iid.equals(Components.interfaces.nsIMsgWindowCommands))
+    if (iid.equals(Components.interfaces.nsIMsgWindowCommands) ||
+        iid.equals(Components.interfaces.nsISupports))
       return this;
     throw Components.results.NS_NOINTERFACE;
-        return null;
   },
   SelectFolder: function(folderUri)
   {
@@ -420,10 +425,10 @@ nsMessagePaneController.prototype =
 {
   QueryInterface : function(iid)
   {
-    if(iid.equals(Components.interfaces.nsIMsgMessagePaneController))
+    if (iid.equals(Components.interfaces.nsIMsgMessagePaneController) ||
+        iid.equals(Components.interfaces.nsISupports))
       return this;
     throw Components.results.NS_NOINTERFACE;
-        return null;
   },
   clearMsgPane: function()
   {
@@ -451,7 +456,7 @@ function loadStartPage() {
                 // first, clear out the charset setting.
                 messenger.setDisplayCharset("");
 
-                window.frames["messagepane"].location = startpage;
+                GetMessagePaneFrame().location = startpage;
                 //dump("start message pane with: " + startpage + "\n");
                 ClearMessageSelection();
             }
@@ -588,19 +593,81 @@ function SetKeywords(aKeywords)
     return;
 
   // these are the UI elements who care about keywords
-  var ids = ["expandedKeywordImage","expandedHeaderView","msgHeaderView","collapsedHeaderView","collapsedKeywordImage","editMessageBox","expandedAttachmentBox"];
-  for (i in ids) {
-    var element = document.getElementById(ids[i]);
-    if (element) {
-      if (aKeywords) 
-        element.setAttribute("class", aKeywords);
-      else {
-        // if no keywords, reset class to the original class
-        element.setAttribute("class", element.getAttribute("originalclass"));
+  var elements = document.getElementsByAttribute("keywordrelated","true");
+  var len = elements.length;
+  for (var i=0; i<len; i++) {
+    var element = elements[i];
+    var originalclass = element.getAttribute("originalclass");
+
+    // we use XBL for certain headers.
+    // if the element has keywordrelated="true"
+    // but no original class, it's an XBL widget
+    // so to get the real element, use getAnonymousElementByAttribute()
+    if (!originalclass) {
+      element = document.getAnonymousElementByAttribute(element, "keywordrelated", "true");
+      originalclass = element.getAttribute("originalclass");
+    }
+
+    if (aKeywords) {
+      if (element.getAttribute("appendoriginalclass") == "true") {
+        aKeywords += " " + originalclass;
       }
+      element.setAttribute("class", aKeywords);
+    }
+    else {
+      // if no keywords, reset class to the original class
+      element.setAttribute("class", originalclass);
     }
   }
 
   // cache the keywords 
   gLastKeywords = aKeywords;
 }
+
+function ShowHideToolBarButtons()
+{
+  var prefBase = "mail.toolbars.showbutton.";
+  var prefBranch = prefServices.getBranch(prefBase);
+  var prefCount = {value:0};
+  var prefArray = prefBranch.getChildList("" , prefCount);
+
+  if (prefArray && (prefCount.value > 0)) {
+    for (var i=0;i < prefCount.value;i++) {
+      document.getElementById("button-" + prefArray[i]).hidden = !(prefBranch.getBoolPref(prefArray[i]));
+    }
+  }
+}
+  
+function AddToolBarPrefListener()
+{
+  try {
+    var pbi = pref.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
+    pbi.addObserver(gMailToolBarPrefListener.domain, gMailToolBarPrefListener, false);
+  } catch(ex) {
+    dump("Failed to observe prefs: " + ex + "\n");
+  }
+}
+
+function RemoveToolBarPrefListener()
+{
+  try {
+    var pbi = pref.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
+    pbi.removeObserver(gMailToolBarPrefListener.domain, gMailToolBarPrefListener);
+  } catch(ex) {
+    dump("Failed to remove pref observer: " + ex + "\n");
+  }
+}
+
+// Pref listener constants
+const gMailToolBarPrefListener =
+{
+  domain: "mail.toolbars.showbutton",
+  observe: function(subject, topic, prefName)
+  {
+    // verify that we're changing a button pref
+    if (topic != "nsPref:changed")
+      return;
+
+    document.getElementById("button-" + prefName.substr(this.domain.length+1)).hidden = !(pref.getBoolPref(prefName));
+  }
+};
